﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using FrienDev.BusinessRules;
using FrienDev.Data;

public partial class Controls_Club_ClubPostsList : System.Web.UI.UserControl
{
    private int m_PageSize = 20;
    private int m_Count;
    protected void Page_Load(object sender, EventArgs e)
    {
        this.gv_ClubPostsList.PreRender+=new EventHandler(gv_ClubPostsList_PreRender);
        BindData();
    }
    void BindData()
    {
        m_Count = ClubRules.Instance.GetPagingCurrentUserClubTopicsCount(ClubId);
        //显示Pager
        if (m_Count == 0)
        {
            this.Pager.Visible = false;
        }
        else
        {
            this.Pager.RecordCount = m_Count;
            m_PageSize = this.Pager.PageSize;
            this.Pager.Visible = true;
        }
        this.gv_ClubPostsList.DataSource = ClubRules.Instance.GetPagingCurrentUserClubTopics(ClubId, (this.Pager.CurrentPageIndex - 1) * m_PageSize, m_PageSize);
        this.gv_ClubPostsList.DataBind();
        if (ClubRules.Instance.GetIsAdminOfClubByUserIdAndClubId(ClubId, UserRules.Instance.GetCurrentUserId()))
        {
            this.gv_ClubPostsList.Columns[4].Visible = true;
        }
    }

    public int ClubId
    {
        get
        {
            return GetQueryStringValue("clubId", 0);
        }
    }

    public void gv_ClubPostsList_PreRender(object sender, EventArgs e)
    {
        BindData();
    }


    protected void gv_ClubPostsList_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            Post post = e.Row.DataItem as Post;
            HyperLink temp_hyPostTitle = e.Row.FindControl("hy_PostTitle") as HyperLink;
            temp_hyPostTitle.Text = post.Title;
            temp_hyPostTitle.NavigateUrl = ClubRules.Instance.GetClubPostUrl(post.PostId,ClubId);
            Label temp_lblShortDate = e.Row.FindControl("lbl_ShortDate") as Label;
            temp_lblShortDate.Text = post.CreateDateTime.ToString("MM-dd");
            Label temp_lblReplyCount = e.Row.FindControl("lbl_ReplyCount") as Label;
            temp_lblReplyCount.Text = ClubRules.Instance.GetClubReplyCountAndViewed(post.PostId, post.Viewed.Value);
            HyperLink temp_hyAuthor = e.Row.FindControl("hy_Author") as HyperLink;
            temp_hyAuthor.Text = post.User.FullName;
            temp_hyAuthor.NavigateUrl = ApiHelper.Instance.GetUserHomepageLink(post.UserId);
            LinkButton temp_lbtDelete = e.Row.FindControl("lbt_Delete") as LinkButton;
            temp_lbtDelete.CommandArgument = post.PostId.ToString();
            LinkButton temp_lbtTopIt = e.Row.FindControl("lbt_TopIt") as LinkButton;
            temp_lbtTopIt.CommandArgument = post.PostId.ToString();
            if (post.IsTop)
            {
                temp_lbtTopIt.Text = "取消置顶"; 
            }
            if (post != null)
            {
                Post lastReplyPost = ClubRules.Instance.GetReplyPosts(post.ParentId.Value).First();
                HyperLink temp_hyReplyAuthor = e.Row.FindControl("hy_PostReplyAuthor") as HyperLink;
                temp_hyReplyAuthor.Text = lastReplyPost.User.FullName;
                temp_hyReplyAuthor.NavigateUrl = ApiHelper.Instance.GetUserHomepageLink(lastReplyPost.UserId);
                Label temp_lblReplyDateTime = e.Row.FindControl("lbl_ReplyDateTime") as Label;
                temp_lblReplyDateTime.Text = lastReplyPost.CreateDateTime.ToString("MM-dd HH:mm");
            }
        }

    }

    protected void gv_ClubPostsList_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {

    }

    protected void gv_ClubPostsList_RowCommand(object sender, GridViewCommandEventArgs e)
    {
        if (e.CommandName == "Delete")
        {
            string postId = e.CommandArgument.ToString();
            ClubRules.Instance.DeleteSessionPostByPostId(postId.ToString());
            BindData();
        }
        if (e.CommandName == "TopIt")
        {
            string postId = e.CommandArgument.ToString();
            ClubRules.Instance.TopClubPost(postId);
            BindData();
        }
    }


    #region Protected Method
    protected int GetQueryStringValue(string QueryName, int nullValue)
    {

        if (string.IsNullOrEmpty(this.Page.Request.QueryString[QueryName]))
        {
            return nullValue;
        }
        return Convert.ToInt32(this.Page.Request.QueryString[QueryName]);
    }
    protected string GetQueryStringValue(string QueryName, string nullValue)
    {

        if (string.IsNullOrEmpty(this.Page.Request.QueryString[QueryName]))
        {
            return nullValue;
        }
        return this.Page.Request.QueryString[QueryName];
    }

    protected bool GetQueryStringValue(string QueryName, bool nullValue)
    {
        if (string.IsNullOrEmpty(this.Page.Request.QueryString[QueryName]))
        {
            return nullValue;
        }
        return Convert.ToBoolean(this.Page.Request.QueryString[QueryName]);
    }
    #endregion

}
